==> enterprisemodules-ora_config-3.9.3.tar.gz: enterprisemodules-ora_config-3.9.3/lib/puppet/type/ora_service/preferred_instances.rb <==
#
# See the file "LICENSE" for the full license governing this code.
#
newproperty(:preferred_instances, :array_matching => :all) do
  include Puppet_X::EnterpriseModules::Oracle::ServiceProperty
  include EasyType::ArrayProperty

  data_type("Optional[String[1]]")

  desc <<-EOD
    A list of preferred instances on which the service runs when the database is administrator managed.

    Here is an example on how to use this:

        ora_service{'my_service':
          ...
          prefered_instances => ['SID1', 'SID2'],
          ...
        }

    When an instance is specified that does not or not yes exist in the cluster, Puppet will
    ignore it.

    This is a cluster only property. On single node database this property will be ignored. If you use it,
    Puppet will issue a warning.

  EOD

  column_name 'Preferred instances'

  defaultto do
    begin
      cluster_instances
    rescue RuntimeError
      # When the database doesn't exist yet, use a nil. Later we will fix the
      # default value back again.
      []
    end
  end

  def insync?(is)
    is = [] if is.to_s == 'absent'
    if cluster?
      is.sort == (should - unknown_instances).sort
    else
      Puppet.info "#{path}: #{name} has no effect on non clustered service." unless should == [resource.sid]
      true
    end
  end

  to_translate_to_resource do |raw_resource|
    value = super(raw_resource)
    value.nil? ? nil : value.to_s.split(',').map(&:strip)
  end

  on_create do
    apply('create')
  end

  on_modify do
    apply('update')
  end

  private

  def apply(type)
    if cluster?
      #
      # If it contains unknown instances at this point in time, filter them out
      #
      if unknown_instances?
        sane_value = value - unknown_instances
        if sane_value.any?
          Puppet.info "#{resource}: Skipping unknown instances #{unknown_instances.join(',')} continue with #{sane_value.join(',')}" unless @warned
          @warned = true
          provider.send("#{type}_#{self.class.name}", sane_value)
        else
          fail "#{resource}: No known instances specified for property preferred_instances"
        end
      else
        provider.send("#{type}_#{self.class.name}", value) if value.any?
      end
    else
      Puppet.warning "#{resource}: preferred_instances has no effect on non clustered service." unless resource.preferred_instances == [resource.sid]
    end
  end

  #
  # If the value is empty, use all current instances as default
  #
  def requested_value
    value.empty? ? cluster_instances : value
  end

  def unknown_instances?
    unknown_instances.any?
  end

  def unknown_instances
    requested_value - cluster_instances
  end

  def known_instances
    requested_value & cluster_instances
  end
end

==> enterprisemodules-ora_config-3.9.4.tar.gz: enterprisemodules-ora_config-3.9.4/lib/puppet/type/ora_service/preferred_instances.rb <==
#
# See the file "LICENSE" for the full license governing this code.
#
newproperty(:preferred_instances, :array_matching => :all) do
  include Puppet_X::EnterpriseModules::Oracle::ServiceProperty
  include EasyType::ArrayProperty

  data_type("Optional[Variant[String[1],Array]]")

  desc <<-EOD
    A list of preferred instances on which the service runs when the database is administrator managed.

    Here is an example on how to use this:

        ora_service{'my_service':
          ...
          prefered_instances => ['SID1', 'SID2'],
          ...
        }

    When an instance is specified that does not or not yes exist in the cluster, Puppet will
    ignore it.

    This is a cluster only property. On single node database this property will be ignored. If you use it,
    Puppet will issue a warning.

  EOD

  column_name 'Preferred instances'

  defaultto do
    begin
      cluster_instances
    rescue RuntimeError
      # When the database doesn't exist yet, use a nil. Later we will fix the
      # default value back again.
      []
    end
  end

  def insync?(is)
    is = [] if is.to_s == 'absent'
    if cluster?
      is.sort == (should - unknown_instances).sort
    else
      Puppet.info "#{path}: #{name} has no effect on non clustered service." unless should == [resource.sid]
      true
    end
  end

  to_translate_to_resource do |raw_resource|
    value = super(raw_resource)
    value.nil? ? nil : value.to_s.split(',').map(&:strip)
  end

  on_create do
    apply('create')
  end

  on_modify do
    apply('update')
  end

  private

  def apply(type)
    if cluster?
      #
      # If it contains unknown instances at this point in time, filter them out
      #
      if unknown_instances?
        sane_value = value - unknown_instances
        if sane_value.any?
          Puppet.info "#{resource}: Skipping unknown instances #{unknown_instances.join(',')} continue with #{sane_value.join(',')}" unless @warned
          @warned = true
          provider.send("#{type}_#{self.class.name}", sane_value)
        else
          fail "#{resource}: No known instances specified for property preferred_instances"
        end
      else
        provider.send("#{type}_#{self.class.name}", value) if value.any?
      end
    else
      Puppet.warning "#{resource}: preferred_instances has no effect on non clustered service." unless resource.preferred_instances == [resource.sid]
    end
  end

  #
  # If the value is empty, use all current instances as default
  #
  def requested_value
    value.empty? ? cluster_instances : value
  end

  def unknown_instances?
    unknown_instances.any?
  end

  def unknown_instances
    requested_value - cluster_instances
  end

  def known_instances
    requested_value & cluster_instances
  end
end
